'use strict'

const ArithmeticOperator = require('../smartMethod/arithmeticOperator');
const BasicOperator = require('../smartMethod/basicOperator');
const CastOperator = require('../smartMethod/castOperator');
const ChainOperator = require('../smartMethod/chainOperator');
const ComparisonOperator = require('../smartMethod/comparisonOperator');
const ReadOperator = require('../smartMethod/readOperator');
const UtilOperator = require('../smartMethod/utilOperator');
const WriteOperator = require('../smartMethod/writeOperator');

module.exports = class ABI {

    // Contract operators
    static Condition(abi, err_msg = 'Conditional error') {
        return [ abi, err_msg ];
    }

    static Method(func, vars = []) {
        try {
            const method = func.split('_').map(string => string.charAt(0).toUpperCase() + string.slice(1)).join('');
            func = method.split('').map(char => {
                if(char === char.toUpperCase()) return '_' + char.toLowerCase();
                else return char;
            }).join('').substring(1);

            if (typeof this[method] !== "function") throw `ABI.${method} doesn\'t exist. `;
            let abi = {};
            const key = '$' + func;
            abi[key] = vars;

            return abi;
        } catch (e) {
            throw e;
        }
    }

    static Param(vars) {
        if (typeof vars === 'string') {
            return {"$load_param": [ vars ] };
        } else if (Array.isArray(vars)) {
            return {"$load_param": vars };
        } else {
            return vars;
        }
    }

    // Arithmetic Operator
    static Add = ArithmeticOperator.add;
    static Sub = ArithmeticOperator.sub;
    static Mul = ArithmeticOperator.mul;
    static Div = ArithmeticOperator.div;
    static PreciseAdd = ArithmeticOperator.precise_add;
    static PreciseSub = ArithmeticOperator.precise_sub;
    static PreciseMul = ArithmeticOperator.precise_mul;
    static PreciseDiv = ArithmeticOperator.precise_div;
    static Scale = ArithmeticOperator.scale;

    // Basic operators
    static If = BasicOperator.if;
    static And = BasicOperator.and;
    static Or = BasicOperator.or;
    static Get = BasicOperator.get;

    // Cast operators
    static GetType = CastOperator.get_type;
    static IsNumeric = CastOperator.is_numeric;
    static IsInt = CastOperator.is_int;
    static IsString = CastOperator.is_string;
    static IsNull = CastOperator.is_null;
    static IsBool = CastOperator.is_bool;
    static IsArray = CastOperator.is_array;
    static IsDouble = CastOperator.is_double;

    // Chain operators
    static GetBlock = ChainOperator.get_block;
    static ListBlock = ChainOperator.list_block;
    static BlockCount = ChainOperator.block_count;
    static GetTransaction = ChainOperator.get_transaction;
    static ListTransaction = ChainOperator.list_transaction;
    static TransactionCount = ChainOperator.transaction_count;
    static GetCode = ChainOperator.get_code;
    static ListCode = ChainOperator.list_code;
    static CodeCount = ChainOperator.code_count;
    static BlockExists = ChainOperator.block_exists;
    static IsValidator = ChainOperator.is_validator;
    static ListUniversal = ChainOperator.list_universal;
    static CountUniversal = ChainOperator.count_universal;
    static GetResourceBlock = ChainOperator.get_resource_block;
    static ListResourceBlock = ChainOperator.list_resource_block;
    static ResourceBlockCount = ChainOperator.resource_block_count;
    static GetBlocks = ChainOperator.get_blocks;
    static GetResourceBlocks = ChainOperator.get_resource_blocks;

    // Comparison operators
    static Eq = ComparisonOperator.eq;
    static Ne = ComparisonOperator.ne;
    static Gt = ComparisonOperator.gt;
    static Lt = ComparisonOperator.lt;
    static Gte = ComparisonOperator.gte;
    static Lte = ComparisonOperator.lte;
    static In = ComparisonOperator.in;

    // Read operators
    static LoadParam = ReadOperator.load_param;
    static ReadUniversal = ReadOperator.read_universal;
    static ReadLocal = ReadOperator.read_local;

    // Util operators
    static Concat = UtilOperator.concat;
    static Strlen = UtilOperator.strlen;
    static RegMatch = UtilOperator.reg_match;
    static EncodeJson = UtilOperator.encode_json;
    static DecodeJson = UtilOperator.decode_json;
    static Hash = UtilOperator.hash;
    static ShortHash = UtilOperator.short_hash;
    static IdHash = UtilOperator.id_hash;
    static SignVerify = UtilOperator.sign_verify;

    // Write operators
    static WriteUniversal = WriteOperator.write_universal;
    static WriteLocal = WriteOperator.write_local;
}